package Device::WWN::Carp;
use strict; use warnings;
our $VERSION = '1.00';
use Carp::Clan::Share;
1;
__END__

=head1 NAME

Device::WWN::Carp - Carp::Clan::Share subclass for Device::WWN

=head1 DESCRIPTION

See L<Carp::Clan::Share>.

=head1 MODULE HOME PAGE

The home page of this module is
L<http://www.jasonkohles.com/software/device-wwn>.  This is where you can
always find the latest version, development versions, and bug reports.  You
will also find a link there to report bugs.

=head1 SEE ALSO

L<http://www.jasonkohles.com/software/device-wwn>

L<Device::WWN|Device::WWN>

=head1 AUTHOR

Jason Kohles C<< <email@jasonkohles.com> >>

L<http://www.jasonkohles.com>

=head1 COPYRIGHT AND LICENSE

Copyright 2008, 2009 Jason Kohles

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

